
# q-expansion of newform 3744.2.m.b, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3744
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[703, [1, 0]], [2341, [-1, 0]], [2081, [1, 0]], [2017, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [-1, 0], [0, -3], [2, 0], [-3, -2], [-3, 0], [0, 0], [-6, 0], [0, 6], [0, 0], [3, 0], [0, 10], [0, 9], [0, -7], [0, -6], [10, 0], [0, 10], [12, 0], [0, -5], [0, 6], [0, 0], [-16, 0], [0, -4], [0, 18], [0, 0], [-4, 0], [0, -12], [15, 0], [6, 0], [-18, 0], [0, 15], [0, 2], [0, -9], [10, 0], [0, 15], [0, 18], [6, 0], [0, 8], [0, 24], [0, 9], [0, 0], [-18, 0], [0, -24], [-23, 0], [0, 0], [0, 5], [0, -21], [-2, 0], [-15, 0], [-9, 0], [0, 19], [0, 0], [0, 0], [-3, 0], [-6, 0], [0, 6], [0, 15], [0, 18], [0, -20], [0, 4], [1, 0], [-18, 0], [12, 0], [9, 0], [2, 0], [-12, 0], [-27, 0], [0, -27], [-15, 0], [0, -16], [0, 4], [-28, 0], [0, -4], [30, 0], [0, 1], [0, 6], [18, 0], [0, 10], [0, -6], [0, 9], [-3, 0], [0, -5], [9, 0], [10, 0], [0, -9], [0, -14], [0, -12], [-7, 0], [0, 24], [0, -12], [0, 29], [0, 12], [0, 15], [-30, 0], [24, 0], [10, 0], [3, 0], [0, -36], [-33, 0], [0, 27], [-23, 0], [0, 21], [-15, 0], [0, -5], [0, 18], [-2, 0], [0, 14], [-30, 0], [27, 0], [22, 0], [-6, 0], [0, -8], [30, 0], [0, -45], [18, 0], [-24, 0], [18, 0], [0, 24], [0, -36], [-18, 0], [19, 0], [0, 12], [4, 0], [-12, 0], [0, -30], [30, 0], [0, 0], [-18, 0], [-51, 0], [-30, 0], [0, -19], [-22, 0], [0, 38], [0, -10], [0, -36], [-49, 0], [-18, 0], [0, 12], [15, 0], [-12, 0], [43, 0], [-14, 0], [-32, 0], [0, -16], [0, -16], [9, 0], [42, 0], [0, -4], [0, -49], [33, 0], [33, 0], [0, 29], [-42, 0], [0, 17], [42, 0], [0, 0], [0, 56], [18, 0], [-17, 0], [-2, 0], [21, 0], [0, 57], [0, -45], [0, -28], [0, 11], [-52, 0], [0, 18], [0, -6], [0, 24], [20, 0], [27, 0], [-18, 0], [0, 36], [40, 0], [-45, 0], [-12, 0], [-22, 0], [56, 0], [0, -26], [0, -48], [0, 60], [0, -14], [0, 22], [0, -44], [0, 6], [0, 18], [-54, 0], [0, -36], [0, -40], [-11, 0], [-16, 0], [0, -45], [38, 0], [0, -57], [6, 0], [0, 30], [0, -44], [0, 52], [-66, 0], [35, 0], [28, 0], [3, 0], [-35, 0], [40, 0], [0, 42], [0, 51], [0, 36], [0, -34], [0, -5], [18, 0], [0, -30], [36, 0], [38, 0], [0, -1], [0, 0], [-18, 0], [-57, 0], [0, -32], [-19, 0], [0, 0], [0, -9], [0, -44], [0, -21], [0, 63], [0, 54], [-9, 0], [30, 0], [-18, 0], [52, 0], [0, 36], [0, 36], [0, 0], [-27, 0], [0, 9], [0, 43], [0, -6], [31, 0], [0, -36], [-18, 0], [-16, 0], [0, 25], [-54, 0], [-15, 0], [0, 44], [60, 0], [0, -3], [-28, 0], [-30, 0], [54, 0], [33, 0], [0, -50], [0, 53], [-45, 0], [0, -36], [-40, 0], [0, 20], [42, 0], [0, -48], [0, 18], [36, 0], [0, 3], [-45, 0], [0, 36], [0, -28], [0, 1], [5, 0], [0, 70], [36, 0], [0, 54], [0, 50], [-78, 0], [0, 6], [-40, 0], [18, 0], [0, -36], [8, 0], [30, 0], [0, -30], [0, 75], [24, 0], [0, 15], [-72, 0], [42, 0], [-18, 0], [-48, 0], [9, 0], [17, 0], [0, 21], [-30, 0], [0, 0], [0, 3], [0, 44], [82, 0], [0, 36], [0, 36], [-62, 0], [0, -36], [0, -21], [12, 0], [59, 0], [67, 0], [-20, 0]]
